﻿using ProjectElectricLocomotive.Entities;
using ProjectElectricLocomotive;
using ProjectElectricLocomotive.MovementStrategy;

namespace ProjectElectricLocomotive.DrawningObjects
{
    /// <summary>
    /// Класс, отвечающий за прорисовку и перемещение объекта-сущности
    /// </summary>
    public class DrawningLocomotive
    {
        /// <summary>
        /// Класс-сущность
        /// </summary>
        public EntityLocomotive? EntityLocomotive { get; protected set; }
        /// <summary>
        /// Ширина окна
        /// </summary>
        protected int _pictureWidth;
        /// <summary>
        /// Высота окна
        /// </summary>
        protected int _pictureHeight;
        /// <summary>
        /// Левая координата прорисовки автомобиля
        /// </summary>
        protected int _startPosX;
        /// <summary>
        /// Верхняя кооридната прорисовки автомобиля
        /// </summary>
        protected int _startPosY;
        /// <summary>
        /// Ширина прорисовки автомобиля
        /// </summary>
        protected readonly int _locomotiveWidth = 120;
        /// <summary>
        /// Высота прорисовки автомобиля
        /// </summary>
        protected readonly int _locomotiveHeight = 70;

        /// <summary>
        /// Координата X объекта
        /// </summary>
        public int GetPosX => _startPosX;
        /// <summary>
        /// Координата Y объекта
        /// </summary>
        public int GetPosY => _startPosY;
        /// <summary>
        /// Ширина объекта
        /// </summary>
        public int GetWidth => _locomotiveWidth;
        /// <summary>
        /// Высота объекта
        /// </summary>
        public int GetHeight => _locomotiveHeight;

        /// <summary>
        /// Получение объекта IMoveableObject из объекта DrawningLocomotive
        /// </summary>
        public IMoveableObject GetMoveableObject => new DrawningObjectLocomotive(this);


        /// <summary>
        /// Конструктор
        /// </summary>
        /// <param name="speed">Скорость</param>
        /// <param name="weight">Вес</param>
        /// <param name="bodyColor">Основной цвет</param>
        /// <param name="width">Ширина картинки</param>
        /// <param name="height">Высота картинки</param>
        /// <returns>true - объект создан, false - проверка не пройдена, нельзя создать объект в этих размерах</returns>

        public DrawningLocomotive(int speed, double weight, Color bodyColor, int width, int height)
        {
            if (width < _locomotiveWidth || height < _locomotiveHeight)
            {
                return;
            }
            _pictureWidth = width;
            _pictureHeight = height;
            EntityLocomotive = new EntityLocomotive(speed, weight, bodyColor);
        }

        public DrawningLocomotive(int speed, double weight, Color bodyColor,
            int width, int height, int locomotiveWidth, int locomotiveHeight)
        {
            if (width < _locomotiveWidth || height < _locomotiveHeight)
            {
                return;
            }
            _pictureWidth = width;
            _pictureHeight = height;
            _locomotiveWidth = locomotiveWidth;
            _locomotiveHeight = locomotiveHeight;
            EntityLocomotive = new EntityLocomotive(speed, weight, bodyColor);
        }

        /// <summary>
        /// Установка позиции
        /// </summary>
        /// <param name="x">Координата X</param>
        /// <param name="y">Координата Y</param>
        public void SetPosition(int x, int y)
        {
            x = Math.Min(Math.Max(0, x), _pictureWidth - _locomotiveWidth);
            y = Math.Min(Math.Max(0, y), _pictureHeight - _locomotiveHeight);
            _startPosX = x;
            _startPosY = y;
        }
        
        /// <summary>
        /// Прорисовка объекта
        /// </summary>
        /// <param name="g"></param>
        public virtual void DrawTransport(Graphics g)
        {
            if (EntityLocomotive == null)
            {
                return;
            }

            // корпус электровоза
            Pen pen = new(Color.Black, 5);
            Brush brush = new SolidBrush(EntityLocomotive.BodyColor);
            Point[] points = new Point[5];
            points[0] = new Point(_startPosX, _startPosY + 40);
            points[1] = new Point(_startPosX + 10, _startPosY + 20);
            points[2] = new Point(_startPosX + 120, _startPosY + 20);
            points[3] = new Point(_startPosX + 120, _startPosY + 60);
            points[4] = new Point(_startPosX, _startPosY + 60);

            g.FillPolygon(brush, points);
            g.DrawLine(pen,
                new Point(_startPosX, _startPosY + 40),
                new Point(_startPosX + 120, _startPosY + 40));

            // окна
            brush = new SolidBrush(Color.FromArgb(0, 162, 232));
            g.FillRectangle(brush, _startPosX + 12, _startPosY + 24, 30, 11);
            g.FillEllipse(brush, _startPosX + 55, _startPosY + 24, 11, 11);
            g.FillEllipse(brush, _startPosX + 75, _startPosY + 24, 11, 11);
            g.FillEllipse(brush, _startPosX + 95, _startPosY + 24, 11, 11);

            // колёса
            brush = new SolidBrush(Color.Black);
            g.FillEllipse(brush, _startPosX + 10, _startPosY + 55, 15, 15);
            g.FillEllipse(brush, _startPosX + 25, _startPosY + 55, 15, 15);
            g.FillEllipse(brush, _startPosX + 80, _startPosY + 55, 15, 15);
            g.FillEllipse(brush, _startPosX + 95, _startPosY + 55, 15, 15);
        }

        /// <summary>
        /// Изменение размера поля
        /// </summary>
        /// <param name="newSize">Новый размер</param>
        public void SetPictureSize(Size newSize)
        {
            _pictureHeight = newSize.Height;
            _pictureWidth = newSize.Width;
        }


        /// <summary>
        /// Проверка, что объект может переместится по указанному направлению
        /// </summary>
        /// <param name="direction">Направление</param>
        /// <returns>true - можно переместится по указанному направлению</returns>
        public bool CanMove(DirectionType direction)
        {
            if (EntityLocomotive == null)
            {
                return false;
            }
            return direction switch
            {
                //влево
                DirectionType.Left => _startPosX - EntityLocomotive.Step > 0,
                //вверх
                DirectionType.Up => _startPosY - EntityLocomotive.Step > 0,
                // вправо
                DirectionType.Right => _startPosX + _locomotiveWidth + EntityLocomotive.Step < _pictureWidth,
                //вниз
                DirectionType.Down => _startPosY + _locomotiveHeight + EntityLocomotive.Step < _pictureHeight,
                _ => false,
            };
        }
        /// <summary>
        /// Изменение направления перемещения
        /// </summary>
        /// <param name="direction">Направление</param>
        public void MoveTransport(DirectionType direction)
        {
            if (!CanMove(direction) || EntityLocomotive == null)
            {
                return;
            }
            switch (direction)
            {
                //влево
                case DirectionType.Left:
                    _startPosX -= (int)EntityLocomotive.Step;
                    break;
                //вверх
                case DirectionType.Up:
                    _startPosY -= (int)EntityLocomotive.Step;
                    break;
                // вправо
                case DirectionType.Right:
                    _startPosX += (int)EntityLocomotive.Step;
                    break;
                //вниз
                case DirectionType.Down:
                    _startPosY += (int)EntityLocomotive.Step;
                    break;
            }
        }

    }
}
